# Licensed Materials - Property of IBM
# Rational ClearCase
# (C) Copyright IBM Corp. 1996, 2014.  All Rights Reserved.
# US Government Users Restricted Rights -
# Use, duplication or disclosure restricted by GSA ADP Schedule
# Contract with IBM Corp
#

require "vob_snap_machdep.pl";


$PARAM_PROGRAM                    = "vob_snapshot_setup";
$PARAM_LSVOB_SUB_COMMAND          = 'lsvob';
$PARAM_MODPARAM_SUB_COMMAND       = 'modparam';
$PARAM_RMPARAM_SUB_COMMAND        = 'rmparam';
$PARAM_SUCCESS             = 0;
$PARAM_FAILED              = 1;
$PARAM_VOB_PARAMETERS      = 'vob_snapshot_parameters';


# Errors

########################################################################
#
#                    VOB_SNAP_PARAM CONSTANTS
#
########################################################################

$PARAM_E_NO_HOSTNAME               = "Get host name failed.\n";
$PARAM_E_GET_LOCAL_VOBS_FAILED     = 
    "Could not obtain list of local vobs\n";
$PARAM_E_GET_DBID_FAILED           =
    "Could not get the dbid of the local replica for vob %s\n";
$PARAM_E_GET_REP_NAME_FAILED       =
    "Could not get the name of the local replica for vob %s\n";
$PARAM_E_GET_REP_ATTR_FAILED       =
    "Could not get the replica snap attribute for vob %s\n";
$PARAM_E_LSVOB_FAILED              =
    "Could both lsvob vob %s\n";
$PARAM_E_GET_CURRENT_PARAMS_FAILED =
"Could not obtain snap attribute to obtain current parameter settings. You
may try passing in values for all valid parameters to reinitialize the entire
attribute.\n";

$PARAM_P_TAG_NAME                  = 
"No vob tag name was supplied on the command line. Supply one now or type 
quit to exit. 
vob tag (default quit): ";
$PARAM_P_DO_DBCHECK                = 
"Do you want a data base check to be performed on this vob after all snapshot
operations on this host are completed?";
$PARAM_P_NOTIFY                    = 
"Supply a comma seperated list of those users to be notified of events during 
the snapshot of this vob.
(Comma seperated user id list: default %s): ";
$PARAM_P_SNAP_TO_E1 =
"Supply a directory to contain the snapshot data for this vob. The directory must
exist and be writable.
(Full pathname: %s) ";
$PARAM_P_SNAP_TO_E2 =
"The directory to contain the snapped data does not exits. Please supply a 
full directory path or type quit to exit. "; 
$PARAM_P_SNAP_TO_E3 =
"The directory to contain the snapped data is not writable. Please supply a 
dfull irectory path or type quit to exit. ";
$PARAM_P_SNAP_TO_E4 =
"The directory to contain the snapped data must be specified as a full 
path name. Please supply a full directory path now or type quit to exit. ";

$PARAM_E_TAG_INVALID               = 
    "Vob tag is invalid\n";
$PARAM_E_MAKE_ATTRTYPE_FAILED        =
    "Could not make the snap attribute type\n";
$PARAM_E_MAKE_ATTR_FAILED            =
    "Could not make the attribute\n";
$PARAM_E_RM_ATTR_FAILED            =
    "Could not remove the attribute\n";
$PARAM_E_CANT_CREATE_DEFDEL      =
"Cant modify the vob_server.conf file to enabling deferred deletes failed\n";
$PARAM_E_NO_VOBCONF =
"No vob_server.conf file exists for this vob. Deferred deletes cannot be
enabled\n";
$PARAM_CANT_OPEN_DEFDEL = 
"Can not open the vob_server.conf file to enable deferred deletes.\n";
$PARAM_CANT_OPEN_TEMP_FOR_DEFDEL  =
"Can not open a temp file to generate a new vob_server.conf file. Deferred
deletes can not be enabled.\n";
$PARAM_CANT_REMOVE_VOBCONF =
    "Can not remove the vob_server.conf file to enable deferred deletes.\n";
$PARAM_CANT_RECREATE_VOBCONF =
"Can not recreate the vob_server.conf file to enable deferred deletes. The
vob_server.conf file no longer exists.\n";



#
# The subcommand bein executed
#
$param_subcommand              = "";

if($MDEP_MACHINE eq 'NT') {
$LSVOB_USAGE =
"lsvob    [ -short | -long ] [-host hostname] vob-tag-list\n";
$MOD_USAGE = 
"modparam [ -dbcheck <yes | no> ] [ -snap_to <directory> ] vob-tag\n";
$RMPARAM_USAGE = "rmparam vob-tag\n";
}
else {
$LSVOB_USAGE =
"lsvob    [ -short | -long ] [-host hostname]  vob-tag-list\n";
$MOD_USAGE = 
"modparam [ -dbcheck <yes | no> ] [ -notify <userid1,userid2> ] [ -snap_to <directory> ] vob-tag\n";
$RMPARAM_USAGE = 
"rmparam vob-tag\n";
}

$SUB_USAGE =
"Unknown sub-command. Possible sub-commands are:
  lsvob
  modparam
  rmparam
";


#
# param_error()
# Inputs:
#   $errstr 
#      The error string to put out.
# Outputs:
#   None
# Returns:
#   None
# Modifies:
#   None
# Calls:
#   &mdep_error()
# Description:
#   A wrapper for the call to param_error. It adds the program name and
#   error string.
#
sub param_error {

    local($errstr) = @_[0];   # The string to emit.

    print "\n\n$PARAM_PROGRAM encountered an ERROR in $param_cur_function:\n";
    print "$errstr";
}

sub param_cleartool_message {

    local(*command,*msg) = @_;
    local($_);

    print "The failed command:\n";
    print "$command\n";
    print "Cleartool Error Message:\n";
    while($_ = shift(@msg)) {print};
}



#
# Function: param_lsvob
# Inputs:
#   @ARGV minus the subcommand
# Output:
#   None
# Returns
#   None
# References:
#   $param_hostname
# Modifies:
#   
# Calls:
#   &param_error
#   &mdep_command
# Description:
#   Go through the vobs on this host or the one vob passed in and check
#   for the snap attribute. If found output the parameters. Also output
#   some other related information including the replica in the same 
#   format as that used to create the snap directory. It can be used 
#   to identify the directory to pull off tape. Of course if the vob
#   is hosed this won't work because we won't be able to get it.
#
#
sub param_lsvob {

    local($arg);                   # Arg holder
    local($argindx) = 0;           # Arg index
    local($format) = "";           # Short or long format    
    local(@vob_tags);              # Vob tags to process
    local($vob_tag);               # One of them
    local($fstr);                  # General format string
    local($rep_dbid);              # Replica dbid
    local($rep_uuid);              # Replica uuid
    local($rep_name);              # Replica Name
    local($fcmd);                  # Buffer to form get attr command
    local($line);                  # Line buffer for parsing input
    local($command);
    local(@cmdbuf);                # mdep_command return buffer
    local(@local_tags);            # Local tags
    local($param_found);           # Found a parameter attribute
    local($dbcheck);               # Dbcheck paramter
    local($snap_to);               # Snap_to parameter
    local($notify);                # Notify list parameter
    local($status);                # General status
    local($rep_uuid_digits);       # Replica uuid, digits only
    local($fam_uuid_digits);       # Family uuid, digits only
    local($local_storage);         # Local vob storage path
    local($global_storage);        # Global vob storage path
    local($param_count) = 0;
    local($_);

  
    $param_cur_function = "lsvob";

    $vob_arg = "";

    #
    # Get the arguments
    #
    while(($_ = shift) =~ /^-/) {

	if(/^-host/) {
	    $host = shift;
	    if(!$host) {
		print "-host requires hostname\n";
		print $LSVOB_USAGE;
		exit(-1);
	    }
	    next;
	}
	if(/^-short/) {
	    if($format) {
		print "Only one of -short or -long allowed\n";
		print $LSVOB_USAGE;
		exit(-1);
	    }
	    $format = "short"; next;
	}
	if(/^-long/) {
	    if($format) {
		print "Only one of -short or -long allowed\n";
		print $LSVOB_USAGE;
		exit(-1);
	    }
	    $format = "long"; next;
	}
	print "Unknown argument \'$_\'\n";
        print $LSVOB_USAGE;
        exit(-1);
    }

    if($_) {

    	push(@vob_tags,$_);
    }


    while($_ = shift) {

	if(/^-/) {
	    print "Unexpect argument: \'$_\'\n";
	    print $LSVOB_USAGE;
	    exit(-1);
	}
	push(@vob_tags,$_);
    }

    #
    # Set the default format if none was given.
    #
    if(!$format) {$format = "short";}

    if(!$host) {$host = $param_hostname;} 
    if(!scalar @vob_tags) {
        $status = &mdep_get_local_tags(*host,*tags);
        if($status ne $RTN_OK) {

            &param_error("Could not obtain vob list\n");
            exit(-1);
        }
        if(!scalar @tags) {
            exit(0);
        }

	foreach $vob_tag (@tags) {
   	    chop $vob_tag;
	    push(@vob_tags,$vob_tag);
	}
    }

NEXT_VOB:

    foreach $vob_tag (@vob_tags) {
        if(!($MDEP_MACHINE eq 'NT')) {
	#
	# Get the replica uuid
	#
	$command = "$MDEP_CLEARTOOL lsvob -long $vob_tag";
	#
	# Save NLSPATH and set to null for English
	#
	$NLSPATH = $ENV{NLSPATH}; 
	$ENV{NLSPATH} = "";
	if(&mdep_command_s(*command,*cmdbuf)) {
	    while($_ = shift(@cmdbuf)) {
		if(/No matching entries found for vob tag/) {
		    print "\n";
		    print  "VobTag:                  $vob_tag\n";
		    print  "                         Tag not found\n";
		    next NEXT_VOB;
		}
	    }
	    &param_error("Failed to get the replica uuid of $vob_tag\n");
	    &param_cleartool_message(*command,*cmdbuf);
	    next NEXT_VOB;
	}
				
	#
	# ParPse the output for the replica uuid and family uuid
	#
	while($_ = shift(@cmdbuf)) {
	    chop;
	    if(/Vob replica uuid: (.*)/) {
		$rep_uuid = $1; next;
	    }
	    if(/Vob family uuid: (.*)/) {
		$fam_uuid = $1; next;
	    }
	    if(/Vob server access path: (.*)/) {
	        $local_storage = $1; next;
	    }
            if(/  Global path: (.*)/) {
	        $global_path = $1; next;
            }
	}
	
	$command = "$MDEP_CLEARTOOL desc vob:${vob_tag}";
	if(&mdep_command_s(*command,*cmdbuf)) {
	    &param_error("Cleartool describe of vob $vob_tag failed\n");
	    &param_cleartool_message(*command,*cmdbuf);
	    next NEXT_VOB;
	}
	
    	#
    	# Parse the output for the resricted mode.
    	#
    	while($_ = shift(@cmdbuf)) {
       	    chop;
       	    if(/  versioned object base operations are restricted/) {
		next NEXT_VOB;
       	    }
    	}
	
        #
	# The only way to get a replica specific attribute is through
        # cleartool describe but it requires the replica name. The only
        # way to get the name is through the cleartool dump command.
        # In fact two cleartool dump commands. The first one gets the
        # dbid, the second uses the dbid to get the name. Ugh.
	$command = "$MDEP_CLEARTOOL dump vob:${vob_tag}";
	if(&mdep_command_s(*command,*cmdbuf)) {
	    &param_error("Failed to get the replica dbid of $vob_tag\n");
	    &param_cleartool_message(@cmdbuf);
	    next NEXT_VOB;
	}
	    
	#
	# Parse the output for the dbid.
	#
	while($_ = shift(@cmdbuf)) {
	    chop;
  	    if(/this replica dbid=(\d+)/) {
		$rep_dbid = $1; last;
	    }
	}
	
	#
	# Now use the dbiid to get the replica's name
	#
	$command = "$MDEP_CLEARTOOL dump dbid:${rep_dbid}\@${vob_tag}";
	if(&mdep_command_s(*command,*cmdbuf)) {
	    &param_error("Failed to get the replica name of $vob_tag\n");
	    &param_cleartool_message(*command,*cmdbuf);
	    next NEXT_VOB;
	}
	    
	#
	# Parse the output for the name.
	#
	while($_ = shift(@cmdbuf)) {
	    chop;
	    if(/mtype=replica  name=\"(.*)\"/) {
		$rep_name = $1; next;
	    }
	}
	    
	#
	# Now we are ready to check for the attribute.
	#
	$command = sprintf($MDEP_GET_SNAP_ATTR,$SNAP_REP_PARAMETERS,
			      $rep_name,$vob_tag);
	if(&mdep_command_s(*command,*cmdbuf)) {
	    &param_error("Check for the snap attribute failed for $vob_tag\n");
	    &param_cleartool_message(*command,*cmdbuf);
	    next NEXT_VOB;
	}
	    
	#
	# We asked for the attribute only but parse the output anyway.
	#
	$param_found = 0;
	while($_ = shift(@cmdbuf)) {
	    chop;
	    # 
	    # The parse format string is declared in vob_snap_machep.pl
	    #
	    if(/$SNAP_REP_PARAMETERS_PARSE/) {
		$dbcheck = $1;
		$snap_to = $2;
		$notify = $3;
		$param_found = 1;
		last;
	    }
	}
	    
	#
	# Either we didn't fond an attribute.....
	#
	if(!$param_found) {
	    next NEXT_VOB;
	}
        $param_count++;
	#
	# .....Or we did.
	#
	if($format eq "short") {
	    print "$vob_tag\n"; next;
	}
	    

        #
	# Find out if deferred deletes are enabled.
	#
	$defdels = &mdep_check_deferred_del($global_path);
				
	#
	# Convert the uuids to the same format that the snap directory
        # file uses.
	#
	$rep_uuid =~ s/\W//g;
	$fam_uuid =~ s/\W//g;
        print "\n";
	print  "VobTag:                  $vob_tag\n";
        print  "Replica Uuid:            $rep_uuid\n";
	print  "Snap To                  $snap_to\n";
	print  "Dbcheck Enabled:         $dbcheck\n";
        if(!($MDEP_MACHINE eq 'NT')) {
            print  "Notification List:       $notify\n";
        }
	if($defdels eq $RTN_SET) {

	    print  "Deferred Deletes:        Enabled\n";
	
	}
	else {

	    print  "Deferred Deletes:        Disabled\n";
	}
    #
    # Now restore NLSPATH
    #
    $ENV{NLSPATH} = $NLSPATH; # Reset NLSPATH

    } else { # for NT 
        #
        # Get the replica uuid
        #
        $command = "$MDEP_CLEARTOOL lsvob -long $vob_tag";

        # set following env. for tagged massage output
        # for lsvob -l command
        $ENV{'CLEARCASE_ENABLE_ATE_MESSAGES'} = "TRUE";

        if(&mdep_command_s(*command,*cmdbuf)) {
            while($_ = shift(@cmdbuf)) {
                if(/No matching entries found for vob tag/) {
                    print "\n";
                    print  "VobTag:                  $vob_tag\n";
                    print  "                         Tag not found\n";
                    next NEXT_VOB;
                }
            }
            &param_error("Failed to get the replica uuid of $vob_tag\n");
            # if the command fail, set env variable back. We also need to
            # print out untagged error messages, then re-run that command again
            if(exists($ENV{'CLEARCASE_ENABLE_ATE_MESSAGES'})) {
                delete $ENV{"CLEARCASE_ENABLE_ATE_MESSAGES"};
            }
            &mdep_command_s(*command,*cmdbuf);
            &param_cleartool_message(*command,*cmdbuf);
            next NEXT_VOB;
        }

        #
        # Parse the output for the replica uuid and family uuid
        #
        while($_ = shift(@cmdbuf)) {
            chop;
            if(/^TAG_ID:cmd!!\d+!!698:END_ID/) {
                shift(@cmdbuf);
                $_ = shift(@cmdbuf);
                if(/^TAG_VALUES:(.*):END_VALUES/) {
                    $rep_uuid = $1; next;
                }
            }
            if(/^TAG_ID:cmd!!\d+!!697:END_ID/) {
                shift(@cmdbuf);
                $_ = shift(@cmdbuf);
                if(/^TAG_VALUES:(.*):END_VALUES/) {
                    $fam_uuid = $1; next;
                }
            }
            if(/^TAG_ID:cmd!!\d+!!696:END_ID/) {
                shift(@cmdbuf);
                $_ = shift(@cmdbuf);
                if(/^TAG_VALUES:(.*):END_VALUES/) {
                    $local_storage = $1; next;
                }
            }
            if(/^TAG_ID:cmd!!\d+!!687:END_ID/) {
                shift(@cmdbuf);
                $_ = shift(@cmdbuf);
                if(/^TAG_VALUES:(.*):END_VALUES/) {
                    $global_path = $1; next;
                }
            }
        }

        # switch back to non-tagged message
        if(exists($ENV{'CLEARCASE_ENABLE_ATE_MESSAGES'})) {
            delete $ENV{"CLEARCASE_ENABLE_ATE_MESSAGES"};
        }

        $command = "$MDEP_CLEARTOOL desc vob:${vob_tag}";
        if(&mdep_command_s(*command,*cmdbuf)) {
            &param_error("Cleartool describe of vob $vob_tag failed\n");
            &param_cleartool_message(*command,*cmdbuf);
            next NEXT_VOB;
        }

        #
        # Parse the output for the resricted mode.
        #
        while($_ = shift(@cmdbuf)) {
            chop;
            if(/  versioned object base operations are restricted/) {
                next NEXT_VOB;
            }
        }

        #
        # The only way to get a replica specific attribute is through
        # cleartool decribe but it requires the replica name. The only
        # way to get the name is through the cleartool dump command.
        # In fact two cleartool dump commands. The first one gets the
        # dbid, the second uses the dbid to get the name. Ugh.
        # cleratool dump is not offically supported command. It has not
        # localization yet. So we don't need to use tagged message here.
        $command = "$MDEP_CLEARTOOL dump vob:${vob_tag}";
        if(&mdep_command_s(*command,*cmdbuf)) {
            &param_error("Failed to get the replica dbid of $vob_tag\n");
            &param_cleartool_message(@cmdbuf);
            next NEXT_VOB;
        }

        #
        # Parse the output for the dbid.
        #
        while($_ = shift(@cmdbuf)) {
            chop;
            if(/this replica dbid=(\d+)/) {
                $rep_dbid = $1; last;
            }
        }

        #
        # Now use the dbiid to get the replica's name
        #
        $command = "$MDEP_CLEARTOOL dump dbid:${rep_dbid}\@${vob_tag}";
        if(&mdep_command_s(*command,*cmdbuf)) {
            &param_error("Failed to get the replica name of $vob_tag\n");
            &param_cleartool_message(*command,*cmdbuf);
            next NEXT_VOB;
        }

        #
        # Parse the output for the name.
        #
        while($_ = shift(@cmdbuf)) {
            chop;
            if(/mtype=replica  name=\"(.*)\"/) {
                $rep_name = $1; next;
            }
        }

        #
        # Now we are ready to check for the attribute.
        #
        $command = sprintf($MDEP_GET_SNAP_ATTR,$SNAP_REP_PARAMETERS,
                              $rep_name,$vob_tag);
        if(&mdep_command_s(*command,*cmdbuf)) {
            &param_error("Check for the snap attribute failed for $vob_tag\n");
            &param_cleartool_message(*command,*cmdbuf);
            next NEXT_VOB;
        }

        #
        # We asked for the attribute only but parse the output anyway.
        #
        $param_found = 0;
        while($_ = shift(@cmdbuf)) {
            chop;
            #
            # The parse format string is declared in vob_snap_machep.pl
            #
            if(/$SNAP_REP_PARAMETERS_PARSE/) {
                $dbcheck = $1;
                $snap_to = $2;
                $notify = $3;
                $param_found = 1;
                last;
            }
        }

        #
        # Either we didn't fond an attribute.....
        #
        if(!$param_found) {
            next NEXT_VOB;
        }
        $param_count++;
        #
        # .....Or we did.
        #
        if($format eq "short") {
            print "$vob_tag\n"; next;
        }


        #
        # Find out if deferred deletes are enabled.
        #
        $defdels = &mdep_check_deferred_del($global_path);

        #
        # Convert the uuids to the same format that the snap directory
        # file uses.
        #
        $rep_uuid =~ s/\W//g;
        $fam_uuid =~ s/\W//g;
        print "\n";
        print  "VobTag:                  $vob_tag\n";
        print  "Replica Uuid:            $rep_uuid\n";
        print  "Snap To                  $snap_to\n";
        print  "Dbcheck Enabled:         $dbcheck\n";
        if(!($MDEP_MACHINE eq 'NT')) {
            print  "Notification List:       $notify\n";
        }
        if($defdels eq $RTN_SET) {

            print  "Deferred Deletes:        Enabled\n";

        }
        else {

            print  "Deferred Deletes:        Disabled\n";
        }

    } # end of if !NT
    } 
    exit($param_count);
}


$param_deletes_deferred = 0;
$param_attribute_set = 0;
#
# Function: param_modparam
# Inputs:
#   @ARGV minus the subcommand
# Output:
#   None
# Returns
#   None
# References:
#   $param_hostname
# Modifies:
#   None
# Calls:
#   &param_fatal
#   &mdep_command
#   &mdep_prompt
#   &mdep_is_tag_valid
#   
# Description:
#   Modify the snap attribute to contain the passed in parameters.
#
sub param_modparam {

    local($arg);                   # Arg holder
    local($vob) = "";              # Vob tag
    local($dbcheck) = "";          # Dbcheck paramter
    local($snap_to) = "";          # Snap_to parameter
    local($notify) = "";           # Notify list parameter
    local($status);                # General return status
    local($line);                  # Line buffer for parsing input
    local($rep_dbid);              # Replica dbid
    local(@cmdbuf);                # mdep_command return buffer
    local($rep_name);              # Replica Name
    local($fcmd);                  # Buffer to form get attr command
    local($param_found);           # Found a parameter attribute
    local($replace);               # Set if the attribute is replaced
    local($attribute);             # Buffer to make the attribute
    local($local_storage);         # Local storage directory
    local($status) = 0;
    local($fstr);
    local($default_notify) = 'no one';
    local($default_dbcheck) = 'no';
    local($default_snap_to_prompt) = 'there is no default';
    local($default_snap_to);
    local($_);
MOD_CLEANUP: {
    $param_cur_function = "modparam";
    $need_dbcheck = 1;
    $need_snap_to = 1;
    if($MDEP_MACHINE eq 'NT') {
	$need_notify = 0;
    }
    else {
	$need_notify = 1;
    }
    
    #
    # Parse the input
    #
    while(($_ = shift) =~ /^-/) {

        if(!($MDEP_MACHINE eq 'NT')) {
	    if(/^-notify/i) {
	        $_ = shift; 
		if(/^-/ || !$_) {
		    print "-notify requires a notification list\n";
		    print $MOD_USAGE;
		    last MOD_CLEANUP;
		}
		if($notify) {
		    print "multiple notification specifications not allowed\n";
		    print $MOD_USAGE;
		    last MOD_CLEANUP;
		}
		$notify = $_;
		$need_notify = 0;
		next;
	    }
	}
	if(/^-snap_to/) {
	    $_ = shift; 
	    if(/^-/ || !$_) {
		print "-snap_to requires a directory path\n";
		print $MOD_USAGE;
		last MOD_CLEANUP;
	    }
	    if($snap_to) {
		print "multiple snap_to specifications not allowed\n";
		print $MOD_USAGE;
		last MOD_CLEANUP;
	    }
	    $snap_to = $_;
            if($MDEP_MACHINE eq 'NT')  {
                if((!($snap_to =~ /^[a-zA-Z]:/) &&
                    (!($snap_to =~ /^\\\\/)))) {
                    print "\nThe snap to directory must be a full pathname\n";
		    print $MOD_USAGE;
                    last MOD_CLEANUP;
                }
            }
            else {
                if(!($snap_to =~ /^\//)) {
                    print "\nThe snap to directory must be a full pathname\n";
		    print $MOD_USAGE;
                    last MOD_CLEANUP;
                }
            }
            if(! -d $snap_to) {
                print "\nThe directory $snap_to does not exist\n";
		print $MOD_USAGE;
                last MOD_CLEANUP;
           }
           if(! -w $snap_to) {
               print "\nThe directory $snap_to is not writable\n";
	       print $MOD_USAGE;
               last MOD_CLEANUP;
            }
	    $need_snap_to = 0;
	    next;
	}
	if(/^-dbcheck/) {
	    $_ = shift;
	    if(!(/yes/i || /no/i)) {
		print "dbcheck must be either yes or no";
		print $MOD_USAGE;
		last MOD_CLEANUP;
	    }
	    $dbcheck = $_;
	    $need_dbcheck = 0;
	    next;
	}
	
	print "Unknown argument \'$_\'\n";
	print $MOD_USAGE;
	last MOD_CLEANUP;
    }
    if(!$_) {
	print "A vob-tag argument is required\n";
	print $MOD_USAGE;
	last MOD_CLEANUP;
    }

    $vob = $_;

    if(scalar @_) {
	print "Unexpected arguments: ";
	while($_ = shift) {
	    print "\'$_\' ";
	}
	print "\n";
	print "$MOD_USAGE";
	last MOD_CLEANUP;
    }

    $status = &mdep_is_tag_valid($vob,$param_hostname);
    if($status ne $RTN_OK) {

	&param_error($PARAM_E_TAG_INVALID);
        last MOD_CLEANUP;
    }

    #
    # If the vob is in restricted mode don't do anything.
    #
    $command = "$MDEP_CLEARTOOL desc vob:${vob}";
    if(&mdep_command_s(*command,*cmdbuf)) {
	&param_error("Cleartool describe failed\n");
	&param_cleartool_message(*command,*cmdbuf);
	last MOD_CLEANUP;
    }
 
    #
    # Parse the output for the resricted mode.
    #
    while($_ = shift(@cmdbuf)) {
	chop;
	if(/  versioned object base operations are restricted/) {
            &param_error(
             "Functionality not supported on vobs in restricted mode\n");
	    last MOD_CLEANUP;
        }
    }

    $command = "$MDEP_CLEARTOOL lsvob -long $vob";

if(!($MDEP_MACHINE eq 'NT')) {
    #
    # Save NLSPATH and set to null for English
    #
    $NLSPATH = $ENV{NLSPATH}; 
    $ENV{NLSPATH} = "";
    if(&mdep_command_s(*command,*cmdbuf)) {
	&param_error("Failed to get the local storage path\n");
	&param_cleartool_message(*command,*cmdbuf);
	last MOD_CLEANUP;
    }
    #
    # ParPse the output for the replica uuid and family uuid
    #
    while($_ = shift(@cmdbuf)) {
	chop;
	if(/Vob server access path: (.*)/) {
	    $local_storage = $1; next;
	}
    }
    #
    # Now restore NLSPATH
    #
    $ENV{NLSPATH} = $NLSPATH; # Reset NLSPATH
    
} else {

    # set following env. for tagged massage output
    # for lsvob -l command
    $ENV{'CLEARCASE_ENABLE_ATE_MESSAGES'} = "TRUE";

    if(&mdep_command_s(*command,*cmdbuf)) {
        &param_error("Failed to get the local storage path\n");
        # if the command fail, set env variable back. We also need to
        # print out untagged error messages, then re-run that command again
        if(exists($ENV{'CLEARCASE_ENABLE_ATE_MESSAGES'})) {
            delete $ENV{"CLEARCASE_ENABLE_ATE_MESSAGES"};
        }
        &mdep_command_s(*command,*cmdbuf);
        &param_cleartool_message(*command,*cmdbuf);
        last MOD_CLEANUP;
    }
    #
    # ParPse the output for the replica uuid and family uuid
    #
    while($_ = shift(@cmdbuf)) {
        chop;
        if(/^TAG_ID:cmd!!\d+!!696:END_ID/) {
            shift(@cmdbuf);
            $_ = shift(@cmdbuf);
            if(/^TAG_VALUES:(.*):END_VALUES/) {
                $local_storage = $1; next;
            }
        }
    }

    # switch back to non-tagged message
    if(exists($ENV{'CLEARCASE_ENABLE_ATE_MESSAGES'})) {
        delete $ENV{"CLEARCASE_ENABLE_ATE_MESSAGES"};
    }
} # end of if !NT

    #
    # If we can't defer deletes don't proceed. The update function
    # fatals on failure.
    #
    $defdel = &mdep_check_deferred_del($local_storage);
    if($defdel ne $RTN_SET) {
	$status = &param_enable_deferred_del($local_storage);

	if($status < 0) {
	    last MOD_CLEANUP;
	}
	if($status) {
	    $param_deletes_deferred++;
	}
    }


    #
    # The only way to get a replica specific attribute is through
    # cleartool decribe but it requires the replica name. The only
    # way to get the name is through the cleartool dump command.
    # In fact two cleartool dump commands. The first one gets the
    # dbid, the second uses the dbid to get the name. Ugh.
    $command = "$MDEP_CLEARTOOL dump vob:${vob}";
    if(&mdep_command_s(*command,*cmdbuf)) {
	&param_error("Failed to get the replica dbid\n");
	&param_cleartool_message(*command,*cmdbuf);
	last MOD_CLEANUP;
    }
    
    #
    # Parse the output for the dbid.
    #
    while($_ = shift(@cmdbuf)) {
	chop;
	if(/this replica dbid=(\d+)/) {
	    $rep_dbid = $1; last;
	}
    }
    #
    # Now use the dbiid to get the replica's name
    #
    $command = "$MDEP_CLEARTOOL dump dbid:${rep_dbid}\@${vob}";
    if(&mdep_command_s(*command,*cmdbuf)) {
	&param_error("Failed to get the replica name\n");
	&param_cleartool_message(*command,*cmdbuf);
	last MOD_CLEANUP;
    }
    
    #
    # Parse the output for the name.
    #
    while($_ = shift(@cmdbuf)) {
	chop;
	if(/mtype=replica  name=\"(.*)\"/) {
	    $rep_name = $1; next;
	}
    }
    
    #
    # Check for the type.
    #
    $command = "$MDEP_CLEARTOOL lstype  -kind attype -invob ${vob}";
    if(&mdep_command_s(*command,*cmdbuf)) {
	&param_error("Check for attribute type failed\n");
	&param_cleartool_message(*command,*cmdbuf);
	last MOD_CLEANUP;
    }

    while($_ = shift(@cmdbuf)) {
	chop;
	if(/$SNAP_REP_PARAMETERS/) {
	    $param_found = 1;
	    last;
	}
    }


    #
    # None there. Create the type first.
    #
    if(!$param_found) {
	$command = "$MDEP_CLEARTOOL mkattype -shared -nc  $SNAP_REP_PARAMETERS\@$vob";
	if(&mdep_command_s(*command,*cmdbuf)) {
	    &param_error("Failed to make attribute type\n");
	    &param_cleartool_message(*command,*cmdbuf);
	    last MOD_CLEANUP;
	}
    }
    else {
	#
	# The replica already has the attribute. Replace the current 
        # value
	#
	if($need_dbcheck || $need_notify || $need_snap_to) {

	    $command = sprintf($MDEP_GET_SNAP_ATTR,$SNAP_REP_PARAMETERS,
			       $rep_name,$vob);

	    if(!&mdep_command_s(*command,*cmdbuf)) {

		#
		# We asked for the attribute only but parse the output anyway.
		#
		while($_ = shift(@cmdbuf)) {

		    chop;
		    # 
		    # The parse format string is declared in vob_snap_machep.pl
		    #
		    if(/$SNAP_REP_PARAMETERS_PARSE/) {

			if($need_notify) {
			    $default_notify = $3;
			}
			if($need_dbcheck) {
			    $default_dbcheck = $1;
			}
			if($need_snap_to) {
                            $default_snap_to_prompt = sprintf("default %s",$2);
                            $default_snap_to = $2;
			}
			last;
		    }
		}
	    }
	}

	$replace = "-replace";
    }

    if($need_snap_to) {
	$error = sprintf($PARAM_P_SNAP_TO_E1,$default_snap_to_prompt);
    }
    $snap_to_not_checked = 1;
    while($snap_to_not_checked) {

	if($need_snap_to) {

	    $snap_to = &mdep_prompt($error,
				    $MDEP_ANY_RESPONSE,
				    $default_snap_to,
				    $MDEP_NO_HELP);
	    if(lc($snap_to) eq $MDEP_QUIT) {
		last MOD_CLEANUP;
	    }
	    $need_snap_to = 0;
	}
       
	if($MDEP_MACHINE eq 'NT')  {
	    if((!($snap_to =~ /^[a-zA-Z]:/) &&
		(!($snap_to =~ /^\\\\/)))) { 
		$error =  $PARAM_P_SNAP_TO_E4;
		$snap_to = "";
		$need_snap_to = 1;
		next
	    }
	}
	else {
	    if(!($snap_to =~ /^\//)) {
		$error = $PARAM_P_SNAP_TO_E4;
		$snap_to = "";
		$need_snap_to = 1;
		next;
	    }
	}
	if(! -d $snap_to) {
	    $error =  $PARAM_P_SNAP_TO_E2;
	    $snap_to = "";
	    $need_snap_to = 1;
	    next;
	}
	if(! -w $snap_to) {
	    $error =  $PARAM_P_SNAP_TO_E3;
	    $snap_to = "";
	    $need_snap_to = 1;
	    next;
	}

	$snap_to_not_checked = 0;
    } 

    if($need_notify) {
        $fstr = sprintf($PARAM_P_NOTIFY,$default_notify);
	$notify = &mdep_prompt($fstr,
			       $MDEP_ANY_RESPONSE,
			       $default_notify,
			       $MDEP_NO_HELP);
	if(lc($notify) eq $MDEP_QUIT) {
	    
	    last MOD_CLEANUP;
	}
    }

    if($need_dbcheck) {
	while(!$dbcheck)  {
	    $dbcheck = &mdep_prompt($PARAM_P_DO_DBCHECK,
				    $MDEP_YES_OR_NO,
				    $default_dbcheck,
				    $MDEP_NO_HELP);
	    if(lc($dbcheck) eq $MDEP_QUIT) {
		
		last MOD_CLEANUP;
	    }
	    if((lc($dbcheck) ne $MDEP_YES) &&
	       (lc($dbcheck) ne $MDEP_NO)) {
		print "dbcheck must be either yes or no\n";
		$dbcheck = "";
	    }
	}
    }

    #
    # Different platforms handle things like quotes in different ways.
    # This mdep function handles these differences.
    #
    $attribute = &mdep_mk_attribute_string($dbcheck,$snap_to,$notify);

    #
    # Set the attribute.
    #
    $command = "$MDEP_CLEARTOOL mkattr -nc $replace $SNAP_REP_PARAMETERS \\\"${attribute}\\\" replica:${rep_name}\@${vob}";
    if(&mdep_command_s(*command,*cmdbuf)) {
	&param_error("Failed to make attribute\n");
	&param_cleartool_message(*command,*cmdbuf);
	last MOD_CLEANUP;
    }

    $param_attribute_set++;

  } # MOD_CLEANUP

    if(!$param_attribute_set && $param_deletes_deferred) {

	&param_disable_deferred_del($local_storage);
	$param_deletes_deferred = 0;
    }

    if($param_deletes_deferred) {
	print "Deferred Deletes Enabled\n";
    }
    if($param_attribute_set) {
	exit(0);
    }
    else {
	exit(-1);
    }
}


#
# Function: param_rmparam
# Inputs:
#   @ARGV minus the subcommand
# Output:
#   None
# Returns
#   None
# References:
#   $param_hostname
# Modifies:
#   None
# Calls:
#   &param_fatal
#   &mdep_command
#   &mdep_is_tag_valid
# Description:
#   Remove the snap attribute from the local hosts replica of the
#   specified vob.
#
sub param_rmparam {

    local($arg);                   # Arg holder
    local($vob);                   # Vob tag
    local($status);                # General return status
    local(@cmdbuf);                # mdep_command return buffer
    local($command);
    local($line);                  # Line buffer for parsing input
    local($rep_dbid);              # Replica dbid
    local($rep_name);              # Replica Name

    local($dbcheck);               # Dbcheck paramter
    local($snap_to);               # Snap_to parameter
    local($notify);                # Notify list parameter
    local($fcmd);                  # Buffer to form get attr command
    local($param_found);           # Found a parameter attribute
    local($replace);               # Set if the attribute is replaced
    local($_);

    $param_cur_function = "rmparam";
    #
    # Parse the arguments
    #

    $_ = shift;
    if(!$_) {
	print "A vob-tag argument is required\n";
	print $RMPARAM_USAGE;
        exit(-1);
    }

    if(/^-/) {
	print "Unexpected argument: \'$_\'\n";
	print $RMPARAM_USAGE;
	exit(-1);
    }

    $vob = $_;

    if(scalar @_) {
	print "Unexpected arguments: ";
	while($_ = shift) {
	    print "\'$_\' ";
	}
	print "\n";
	print "$MOD_USAGE";
	exit(1);
    }

    $status = &mdep_is_tag_valid($vob,$param_hostname);
    if($status ne $RTN_OK) {

	&param_error($PARAM_E_TAG_INVALID);
        exit(1);
    }

    #
    # If the vob is in restricted mode don't do anything.
    #
    $command = "$MDEP_CLEARTOOL desc vob:${vob}"; 
    if(&mdep_command_s(*command,*cmdbuf)) {
	&param_error("Cleartool describe failed\n");
	&param_cleartool_message(*command,*cmdbuf);
	exit();
    }     

    #
    # Save NLSPATH and set to null for English
    #
    $NLSPATH = $ENV{NLSPATH}; 
    $ENV{NLSPATH} = "";
        
    #
    # Parse the output for the resricted mode.
    #
    while($_ = shift(@cmdbuf)) {
        chop;
        if(/  versioned object base operations are restricted/) {
            &param_error(
             "Functionality not supported on vobs in restricted mode\n");
            exit(1);
        }
    }

    $command = "$MDEP_CLEARTOOL lsvob -long $vob";

if($MDEP_MACHINE eq 'NT') {
    # set following env. for tagged massage output
    # for lsvob -l command
    $ENV{'CLEARCASE_ENABLE_ATE_MESSAGES'} = "TRUE";

    if(&mdep_command_s(*command,*cmdbuf)) {
        &param_error("Failed to get the local storage path\n");
        # if the command fail, set env variable back. We also need to
        # print out untagged error messages, then re-run that command again
        if(exists($ENV{'CLEARCASE_ENABLE_ATE_MESSAGES'})) {
            delete $ENV{"CLEARCASE_ENABLE_ATE_MESSAGES"};
        }
        &mdep_command_s(*command,*cmdbuf);
        &param_cleartool_message(*command,*cmdbuf);
        last MOD_CLEANUP;
    }
    #
    # ParPse the output for the replica uuid and family uuid
    #
    while($_ = shift(@cmdbuf)) {
        chop;
        if(/^TAG_ID:cmd!!\d+!!696:END_ID/) {
            shift(@cmdbuf);
            $_ = shift(@cmdbuf);
            if(/^TAG_VALUES:(.*):END_VALUES/) {
                $local_storage = $1; next;
            }
        }
    }

    # switch back to non-tagged message
    if(exists($ENV{'CLEARCASE_ENABLE_ATE_MESSAGES'})) {
        delete $ENV{"CLEARCASE_ENABLE_ATE_MESSAGES"};
    }

} else {

    if(&mdep_command_s(*command,*cmdbuf)) {
        &param_error("Failed to get the local storage path\n");
        &param_cleartool_message(*command,*cmdbuf);
        exit();
    }
 
    #
    # ParPse the output for the replica uuid and family uuid
    #
    while($_ = shift(@cmdbuf)) {
        chop;
        if(/Vob server access path: (.*)/) {
            $local_storage = $1; next;
        }
    }
 
} # end of if NT

    #
    # If we can't defer deletes don't proceed. The updata function
    # fatals on failure.
    #
    $defdel = &mdep_check_deferred_del($local_storage);
    if($defdel ne $RTN_NOT_SET) {
       &param_disable_deferred_del ($local_storage);
    }

    #
    # The only way to get a replica specific attribute is through
    # cleartool decribe but it requires the replica name. The only
    # way to get the name is through the cleartool dump command.
    # In fact two cleartool dump commands. The first one gets the
    # dbid, the second uses the dbid to get the name. Ugh.
    $command = "$MDEP_CLEARTOOL dump vob:${vob}";
    if(&mdep_command_s(*command,*cmdbuf)) {
        &param_error("Failed to get the replica dbid\n");
        &param_cleartool_message(*command,*cmdbuf);
        exit();
    }
 
    #
    # Parse the output for the dbid.
    #
    while($_ = shift(@cmdbuf)) {
        chop;
        if(/this replica dbid=(\d+)/) {
            $rep_dbid = $1; last;
        }
    }

    
    
    #
    # Now use the dbiid to get the replica's name
    #
    $command = "$MDEP_CLEARTOOL dump dbid:${rep_dbid}\@${vob}";
    if(&mdep_command_s(*command,*cmdbuf)) {
        &param_error("Failed to get the replica name\n");
        &param_cleartool_message(*command,*cmdbuf);
        exit();
    }
 
    #
    # Parse the output for the name.
    #
    while($_ = shift(@cmdbuf)) {
        chop;
        if(/mtype=replica  name=\"(.*)\"/) {
            $rep_name = $1; next;
        }
    }
    #
    # Now restore NLSPATH
    #
    $ENV{NLSPATH} = $NLSPATH; # Reset NLSPATH

    $command = 
"$MDEP_CLEARTOOL rmattr -nc $SNAP_REP_PARAMETERS replica:${rep_name}\@${vob}";
    if(&mdep_command_s(*command,*cmdbuf)) {
	&param_error($PARAM_E_RM_ATTR_FAILED);
        &param_cleartool_message(*command,*cmdbuf);
        exit(1);
    }

}

$MDEP_VOB_SERVER_CONF          = 'vob_server.conf';
$MDEP_DEFFERED_DEL_PARAMETER   = 'DeferredSourceContainerDeletion';



sub param_enable_deferred_del {

    local($storage_dir)= @_[0]; # Replica storage area
    local($confpath);           # Path to vob_server conf file
    local($line);               # Line buffer for parsing file
    local($tmpname);            # Temp file for new conf file generation
    local($found);
    local($ret_status);         # Return status
    local($setit) = 0;


    $confpath  = $storage_dir.$MDEP_SEP.$MDEP_VOB_SERVER_CONF;
    if(! -w $storage_dir) {

	&param_error($PARAM_E_CANT_CREATE_DEFDEL);
        return(-1);
    }

    if(! -e $confpath) {

	&param_error($PARAM_E_NO_VOBCONF);
        return(-1);
    }

    $confpath = $storage_dir.$MDEP_SEP.$MDEP_VOB_SERVER_CONF;
    unless(open(VOB_CONF,$confpath)) {

	&param_error($PARAM_CANT_OPEN_DEFDEL);
        return(-1);

    } 

    $tmpname = &mdep_get_tmpname();
    unless(open(TEMP_FILE,">$tmpname")) {

	&param_error($PARAM_CANT_OPEN_TEMP_FOR_DEFDEL);
        return(-1);
    } 
    $found = 0;
    foreach $line (<VOB_CONF>) {
	if($line =~ /$MDEP_DEFFERED_DEL_PARAMETER=(.*)/) {

            if(lc($1) eq "no") {
		$setit++;
	    }
	    print TEMP_FILE "$MDEP_DEFFERED_DEL_PARAMETER=yes\n";
	    $found = 1;
	    
	}
	else {
	    print TEMP_FILE $line;
	}
    }

    close(VOB_CONF);
    close(TEMP_FILE);
	
    
    $status = &mdep_rmfile($confpath);
    
    if($status ne $RTN_OK) {
	
	&param_error($PARAM_CANT_REMOVE_VOBCONF);
        return(-1);
    }
    
    unless(open(VOB_CONF,">$confpath")) {
	&param_error($PARAM_CANT_RECREATE_VOBCONF);
        return(-1);
    }
    
    unless(open(TEMP_FILE,$tmpname)) {
	&param_error($PARAM_CANT_RECREATE_VOBCONF);
        return(-1);
    }

    foreach $line (<TEMP_FILE>) {
	print VOB_CONF $line;
    }
    if(!$found) {

        print VOB_CONF "$MDEP_DEFFERED_DEL_PARAMETER=yes\n";

    }

    close(VOB_CONF);
    close(TEMP_FILE);
    
    &mdep_rmfile($tmpname);

    return($setit);    
}


sub param_disable_deferred_del {

    local($storage_dir)= @_[0]; # Replica storage area
    local($confpath);           # Path to vob_server conf file
    local($line);               # Line buffer for parsing file
    local($tmpname);            # Temp file for new conf file generation
    local($found);
    local($ret_status);         # Return status


    $confpath  = $storage_dir.$MDEP_SEP.$MDEP_VOB_SERVER_CONF;
    if(! -w $storage_dir) {

	&param_error($PARAM_E_CANT_CREATE_DEFDEL);
        exit(1);
    }

    if(! -e $confpath) {

	&param_error($PARAM_E_NO_VOBCONF);
        exit(1);
    }

    $confpath = $storage_dir.$MDEP_SEP.$MDEP_VOB_SERVER_CONF;

    unless(open(VOB_CONF,$confpath)) {

	&param_error($PARAM_CANT_OPEN_DEFDEL);
        exit(1);

    } 

    $tmpname = &mdep_get_tmpname();
    unless(open(TEMP_FILE,">$tmpname")) {

	&param_error($PARAM_CANT_OPEN_TEMP_FOR_DEFDEL);
        exit(1);
    } 
    $found = 0;
    foreach $line (<VOB_CONF>) {
	if($line =~ /$MDEP_DEFFERED_DEL_PARAMETER=(.*)/) {
            if(lc($1) ne 'no') {

                print "Deferred Deletes Disabled\n";
            }
	    print TEMP_FILE "$MDEP_DEFFERED_DEL_PARAMETER=no\n";
	    $found = 1;
	    
	}
	else {
	    print TEMP_FILE $line;
	}
    }

    close(VOB_CONF);
    close(TEMP_FILE);
	
    
    $status = &mdep_rmfile($confpath);
    
    if($status ne $RTN_OK) {
	
	&param_error($PARAM_CANT_REMOVE_VOBCONF);
        exit(1);
    }
    
    unless(open(VOB_CONF,">$confpath")) {
	&param_error($PARAM_CANT_RECREATE_VOBCONF);
        exit(1);
    }
    
    unless(open(TEMP_FILE,$tmpname)) {
	&param_error($PARAM_CANT_RECREATE_VOBCONF);
        exit(1);
    }

    foreach $line (<TEMP_FILE>) {
	print VOB_CONF $line;
    }
    if(!$found) {

        print VOB_CONF "$MDEP_DEFFERED_DEL_PARAMETER=no\n";
        print "Deferred Deletes Disabled\n";
    }

    close(VOB_CONF);
    close(TEMP_FILE);
    
    &mdep_rmfile($tmpname);
    
}

# CLEARCASE_ENABLE_ATE_MESSAGES environment variable, 
# might cause printing a lot of extra messages 
# which confuse this perl script. This is why we
# need to remove this variable from the 
# ENV array. 
delete $ENV{"CLEARCASE_ENABLE_ATE_MESSAGES"};

$CLEARCASE_LIGHT = "";
$vercmd = "$MDEP_CLEARTOOL -version";

if(&mdep_command_s(*vercmd,*cmdbuf)) {
    &param_error("Could not obtain ClearCase version information\n");
    &param_cleartool_message(*command,*cmdbuf);
    exit(1);
}

while($_ = shift(@cmdbuf)) {
    chop;

    if(/ClearCase LT/) {
        $CLEARCASE_LIGHT = TRUE;
    }
}


#
# Make sure we have a host
#
$param_hostname = &mdep_get_hostname();
if($param_hostname eq "") {

    &param_error($PARAM_E_NO_HOSTNAME);
}

#
# Pick off the command and call the appropriate function passing it the
# remaining arguments.
#
$param_subcommand = shift(@ARGV);
if($param_subcommand eq $PARAM_LSVOB_SUB_COMMAND) {

    &param_lsvob(@ARGV);
}
elsif($param_subcommand eq $PARAM_MODPARAM_SUB_COMMAND) {

    &param_modparam(@ARGV);
}
elsif($param_subcommand eq $PARAM_RMPARAM_SUB_COMMAND) {

    &param_rmparam(@ARGV);
}
else {

    &param_error($SUB_USAGE);
    
}




